@@ -167,7 +167,7 @@ class Prpcrypt(object):
             print(e)
             return  ierror.WXBizMsgCrypt_DecryptAES_Error,None
         try:
-            pad = ord(plain_text.decode()[-1]) 
+            pad = plain_text[-1] 
             # 去掉补位字符串 
             #pkcs7 = PKCS7Encoder()
             #plain_text = pkcs7.encode(plain_text)   
@@ -195,10 +195,10 @@ class WXBizMsgCrypt(object):
     #构造函数
     def __init__(self,sToken,sEncodingAESKey,sReceiveId):
         try:
-            self.key = base64.b64decode(sEncodingAESKey+"=")  
+            self.key = base64.b64decode(sEncodingAESKey+"=")
             assert len(self.key) == 32
         except:
-            throw_exception("[error]: EncodingAESKey unvalid !", FormatException) 
+            throw_exception("[error]: EncodingAESKey unvalid !", FormatException)
             # return ierror.WXBizMsgCrypt_IllegalAesKey,None
         self.m_sToken = sToken
         self.m_sReceiveId = sReceiveId
