@@ -12,6 +12,9 @@ import tempfile
 import os
 import pickle
 import json
+from random import randint
+from urllib.parse import quote
+from dataclasses import dataclass
 
 import requests
 from .WXBizMsgCrypt import (
@@ -70,7 +73,7 @@ class ProviderClient(_QywxBase):
         _tmppath = tempfile.gettempdir()
         if not os.path.isdir(_tmppath):
             raise Exception("template path is invalid")
-        _cache_token_file = os.path.join(_tmppath, f"{self._corpid}_{self._corpsecret}.bin")
+        _cache_token_file = os.path.join(_tmppath, f"{self._corpid}.bin")
         if os.path.isfile(_cache_token_file):
             with open(_cache_token_file, "rb") as f:
                 _catched_data = pickle.load(f)
@@ -150,6 +153,30 @@ class ProviderClient(_QywxBase):
         return ret_msg_struct
 
 
+@dataclass
+class ProviderParam:
+    """
+    服务商sdk构造参数
+    """
+    # 服务供应商企业编号
+    provider_id: str
+    # 服务供应商密钥
+    provider_secret: str
+    token: str
+    aeskey: str
+
+@dataclass
+class SuiteParam:
+    """
+    第三方应用sdk构造参数
+    """
+    provider_id: str
+    provider_secret: str
+    suite_id: str
+    suite_secret: str
+    token: str
+    aeskey: str
+
 
 class ProxySuiteUnit:
     """
@@ -199,6 +226,33 @@ class ProxySuiteUnit:
             )
         return None
 
+    @classmethod
+    def loadProviderClient(cls, param: ProviderParam):
+        """
+        为sdk设置服务商参数
+        """
+        return cls(
+            param.provider_id,
+            param.provider_secret,
+            "", "", "", "",
+            param.token,
+            param.aeskey
+        )
+
+    @classmethod
+    def loadSuiteClient(cls, param: SuiteParam):
+        """
+        为sdk设置应用参数
+        """
+        return cls(
+            param.provider_id, param.provider_secret, 
+            "", "",
+            param.suite_id,
+            param.suite_secret,
+            param.token,
+            param.aeskey
+        )
+    
     def postRequest(self, url: str, params: dict):
         try:
             _response = requests.post(url, bytes(
@@ -250,6 +304,36 @@ class ProxySuiteUnit:
             return result
         return None
 
+    def getPreAuthCode(self):
+        """
+        客户企业获取第三方应用的预授权码，用于企业授权时的第三方服务商安全验证
+        """
+        _url = f"https://qyapi.weixin.qq.com/cgi-bin/service/get_pre_auth_code?suite_access_token={self.getSuiteAccessToken()}&"
+        ret, result = self._provider_client.getRequest(_url, {})
+        if ret:
+            return result
+        else: 
+            logger.error(result)
+            return None
+
+    def genSuiteAuthUrl(self, pre_auth_code: str, redirect_url: str, state: str):
+        """
+        获取“企业微信应用授权”入口网址
+        :param pre_auth_code str 预授权码
+        :param redirect_url str 授权后的重定向地址
+        :param state str 
+        """
+        return f"https://open.work.weixin.qq.com/3rdapp/install?suite_id={self._suite_id}&pre_auth_code={pre_auth_code}&redirect_uri={quote(redirect_url)}&state={state}"
+
+    def genThirdSuiteOauthUrl(self, redirect_url: str, state: str, scope: str = "snsapi_base"):
+        """
+        构造第三方应用网页授权链接
+        :param redirect_url str 重定向地址
+        :param state str
+        :param scope str snsapi_base or snsapi_privateinfo
+        """
+        return f"https://open.weixin.qq.com/connect/oauth2/authorize?appid={self._suite_id}&redirect_uri={quote(redirect_url)}&response_type=code&scope={scope}&state={state}#wechat_redirect"
+
     def _refreshSuiteAccessToken(self):
         """
         获取开发套件的访问凭据
@@ -270,6 +354,8 @@ class ProxySuiteUnit:
                 "exptm": _exp_timestamp,
                 "suite_access_token": _access_token
             }
+        else:
+            logger.error(_token_result)
         return None
 
     def _refreshProxyAccessToken(self, corp_id: str, secret: str):
@@ -427,15 +513,78 @@ class ProxySuiteUnit:
         从oauth2认证码转用户编号
         :param code str oauth2登录认证码
         """
-        _url = f"https://qyapi.weixin.qq.com/cgi-bin/auth/getuserinfo?access_token={self.getAccessToken()}&code={code}&debug=1"
+        _url = f"https://qyapi.weixin.qq.com/cgi-bin/auth/getuserinfo?access_token={self.getAccessToken()}&code={code}"
         logger.debug(_url)
         return self.getRequest(_url, {})
 
+    def getOauth2RedirectUrl(self, url: str, agentid: str, state: str=None):
+        """
+        构造oauth2认证跳转连接
+        :param url: 实际的业务地址
+        :param state: 登录状态码
+        """
+        if not state: state = randint(100, 1000000)
+        _redirectUrl = quote(url)
+        return f"https://open.weixin.qq.com/connect/oauth2/authorize?appid={self._corp_id}&redirect_uri={_redirectUrl}&response_type=code&scope=snsapi_base&state={state}&agentid={agentid}#wechat_redirect"
+
     def getOrganization(self):
         """
         获取组织架构
         """
         _url = f"https://qyapi.weixin.qq.com/cgi-bin/department/list?access_token={self.getAccessToken()}"
+        _ret, _orgnization_tree = self.getRequest(_url, {})
+        _department_tree = []
+        if _ret:
+            # 解析全量组织架构
+            logger.info(_department_tree)
+            _origin_department_list = _orgnization_tree.get("department")
+            _origin_dep_array = {}
+            for _dep in _origin_department_list:
+                _cur_parent_id = _dep["parentid"]
+                if not _cur_parent_id in _origin_dep_array:
+                    _origin_dep_array[_cur_parent_id] = []
+                _origin_dep_array[_cur_parent_id].append(_dep)
+            return True, self._make_orgnization_tree(0, _origin_dep_array)
+        else:
+            return False, _orgnization_tree
+        
+    def _make_orgnization_tree(self, parentid: int, dep_tree: dict):
+        """
+        在内存中构造组织架构树
+        """
+        if parentid in dep_tree:
+            dep_tree[parentid] = sorted(dep_tree[parentid], key=lambda x: x["order"], reverse=True)
+            _level_nodes = []
+            for _dep in dep_tree[parentid]:
+                _dep["children"] = self._make_orgnization_tree(_dep["id"], dep_tree)
+                _level_nodes.append(_dep)
+            return _level_nodes
+        else:
+            return []
+
+
+    def getFullDepartmentTree(self):
+        """
+        获取完整的组织架构树
+        该方法存在循环调用接口的机制，效率极低。
+        """
+        _ret, _dep_tree = self.getDepartmentSimpleList()
+        _department_tree = []
+        if _ret:
+            for _dep_node in _dep_tree["department_id"]:
+                _chk, _depinfor = self.getDepartmentInfor(_dep_node["id"])
+                if _chk:
+                    _department_tree.append(_depinfor["department"])
+        return _department_tree
+
+    def getDepartmentSimpleList(self, depid: int=0):
+        """
+        获取部门的编号及上下级信息
+        """
+        _url = f"https://qyapi.weixin.qq.com/cgi-bin/department/simplelist?access_token={self.getAccessToken()}"
+        logger.debug(_url)
+        if depid:
+            _url += f"&id={depid}"
         return self.getRequest(_url, {})
 
     def getDepartmentInfor(self, depid: int):
@@ -461,6 +610,24 @@ class ProxySuiteUnit:
         _url = f"https://qyapi.weixin.qq.com/cgi-bin/user/get?access_token={self.getAccessToken()}&userid={user_id}"
         return self.getRequest(_url, {})
 
+    def getUserInfor3rdFromAuthCode(self, code: str):
+        """
+        根据code参数获取访问用户身份
+        """
+        _url = f"https://qyapi.weixin.qq.com/cgi-bin/service/auth/getuserinfo3rd?suite_access_token={self.getSuiteAccessToken()}&code={code}"
+        return self.getRequest(_url, {})
+
+    def UserIdListBatch(self, batchsize: int = 100):
+        """
+        批量获取用户编号
+        :param batchsize int 批次大小
+        """
+        _url = f"https://qyapi.weixin.qq.com/cgi-bin/user/list_id?access_token={self.getSuiteAccessToken()}"
+        return self.postRequest(_url, {
+            "cursor": "",
+            "limit": batchsize
+        })
+
     def mobile2UserId(self, mobile: str):
         """
         通过手机号码获取用户账号
